/*
 * ERIKA Enterprise Basic - a tiny RTOS for small microcontrollers
 *
 * Copyright (C) 2002-2007  Evidence Srl
 *
 * This file is part of ERIKA Enterprise Basic.
 *
 * ERIKA Enterprise Basic is free software; you can redistribute it
 * and/or modify it under the terms of the GNU General Public License
 * version 2 as published by the Free Software Foundation.
 *
 * ERIKA Enterprise Basic is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 2 for more details.
 *
 * You should have received a copy of the GNU General Public License
 * version 2 along with ERIKA Enterprise Basic; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301 USA.
 */

/* 
 * Author: 2006 Mauro Marinoni
 * CVS: $Id: lcd-protocol.h,v 1.1 2007/05/25 16:03:51 pj Exp $
 */

#ifndef __INCLUDE_LCD_PROTOCOL_H__
#define __INCLUDE_LCD_PROTOCOL_H__
#include "types.h"
#include "delays.h"
/* 
   For Explorer 16 board, here are the data and control signal definitions
   RS -> RB10
   E   -> RB9
   RW -> N.C.
   DATA -> RA0 - RA7   
*/

/* /\* Control signal data pins *\/ */
#define  EE_LCD_RS		LATBbits.LATB10	// LCD RS signal
#define  EE_LCD_E		LATBbits.LATB9	// LCD Enable signal
// fra
#define  EE_LCD_VLCD		LATBbits.LATB8	// LCD Voltage control
#define  EE_LCD_BRIGHTNESS	LATBbits.LATB11	// LCD Brightness control

/* /\* Control signal pin direction *\/ */
#define  EE_LCD_RS_TRIS		TRISBbits.TRISB10
#define  EE_LCD_E_TRIS		TRISBbits.TRISB9
// fra
#define  EE_LCD_VLCD_TRIS	TRISBbits.TRISB8
#define  EE_LCD_BRIGHTNESS_TRIS	TRISBbits.TRISB11

/* /\* Data signals and pin direction *\/ */
#define  EE_LCD_DATA		LATA           // Port for LCD data
#define  EE_LCD_DATAPORT	PORTA
#define  EE_LCD_TRISDATA	TRISA          // I/O setup for data Port


/* /\* Send an impulse on the enable line.  *\/ */
__inline__ void   EE_lcd_pulse_enable( void )
{
	EE_LCD_E = 1;
	Nop();
	Nop();
	Nop();
	EE_LCD_E = 0;
}

/* /\* Send a command to the lcd.  *\/ */
__inline__ void   EE_lcd_command( EE_UINT8 cmd )
{
	EE_LCD_DATA &= 0xFF00;
	EE_LCD_DATA |= cmd;
	EE_LCD_RS = 0;  // fra
	EE_lcd_pulse_enable();
	Delay_Us( Delay200uS_count );
}

/* /\* Switch on or off the back illumination  *\/ */
__inline__ void   EE_lcd_brightness( EE_UINT8 btns_status ) 
{
	EE_LCD_BRIGHTNESS = btns_status & 1;
}

/* /\* Switch on or off the display  *\/ */
__inline__ void   EE_lcd_switch( EE_UINT8 lcd_status ) 
{
	EE_LCD_VLCD = lcd_status & 1;
}

/* /\* Initialize the display.  *\/ */
__inline__ void   EE_lcd_init(void) {
	// 15mS delay after Vdd reaches nnVdc before proceeding with LCD initialization
	// not alwEE_LCD_VLCDays required and is based on system Vdd rise rate
	// Todo!!!
	EE_LCD_VLCD_TRIS	= 0;
	EE_LCD_BRIGHTNESS_TRIS	= 0;
	EE_LCD_VLCD       = 1;
	EE_LCD_BRIGHTNESS = 1;
	
	/* Use pin as digital IO */
 	AD1PCFGLbits.PCFG9  = 0;
 	AD1PCFGLbits.PCFG10 = 0;
			
	/* Initial values */
	EE_LCD_DATA &= 0xFF00;	
	EE_LCD_RS   = 0;
	EE_LCD_E    = 0;
		
	/* Set pins direction */
	EE_LCD_TRISDATA &= 0xFF00;
	EE_LCD_RS_TRIS  = 0;
	EE_LCD_E_TRIS   = 0;
	Delay( Delay_20mS_Cnt );
	
	// Init - Step 1 
	EE_LCD_DATA &= 0xFF00;
	EE_LCD_DATA |= 0x0038;
	EE_lcd_pulse_enable();
	Delay_Us( Delay500uS_count );
      
	// Init - Step 2 
	EE_LCD_DATA &= 0xFF00;
	EE_LCD_DATA |= 0x0038;
	EE_lcd_pulse_enable();
	Delay_Us( Delay200uS_count );

	// Init - Step 3 
	EE_LCD_DATA &= 0xFF00;
	EE_LCD_DATA |= 0x0038;
	EE_lcd_pulse_enable();
	Delay_Us( Delay200uS_count );

	EE_lcd_command( 0x38 );	// Function set
	EE_lcd_command( 0x0C );	// Display on/off control, cursor blink off (0x0C)
	EE_lcd_command( 0x06 );	// Entry mode set (0x06)
}

/* /\* Send a data.  *\/ */
__inline__ void   EE_lcd_putc( char data )
{
	EE_LCD_RS = 1;
	EE_LCD_DATA &= 0xFF00;
	EE_LCD_DATA |= data;
	EE_lcd_pulse_enable();
	EE_LCD_RS = 0;
	Delay_Us( Delay200uS_count );
}

/* /\* Send a string to the display.  *\/ */
__inline__ void   EE_lcd_puts( char *buf )
{
	EE_UINT8 i = 0;
	
	while (buf[i] != '\0')
		EE_lcd_putc(buf[i++]);
}

/* /\* Check if the display is busy.  *\/ */
__inline__ unsigned char   EE_lcd_busy( void )
{
	EE_INT8 buf;
	
	EE_LCD_TRISDATA |= 0x00FF;
	EE_LCD_RS = 1;
	EE_lcd_pulse_enable();
	//Delay_Us( Delay200uS_count );
	buf = EE_LCD_DATAPORT & 0x00FF;
	EE_LCD_RS = 0;
	EE_LCD_TRISDATA &= 0xFF00;
	return ( (buf & 0x80) ? 1 : 0 );
}

__inline__ void   EE_lcd_clear (void)		{ EE_lcd_command( 0x01 ); }

__inline__ void   EE_lcd_home  (void)		{ EE_lcd_command( 0x02 ); }
__inline__ void   EE_lcd_line2 (void)		{ EE_lcd_command( 0xC0 ); }

__inline__ void   EE_lcd_curs_right (void)	{ EE_lcd_command( 0x14 ); }
__inline__ void   EE_lcd_curs_left (void)	{ EE_lcd_command( 0x10 ); }
__inline__ void   EE_lcd_shift (void)		{ EE_lcd_command( 0x1C ); }

__inline__ void   EE_lcd_goto (EE_UINT8 posx, EE_UINT8 posy)
{
	EE_UINT8 tmp_pos;
	
	tmp_pos  = posy ? 0xC0 : 0x80;
	tmp_pos += 0x0F & posx;
	EE_lcd_command( tmp_pos ); 
}

#endif
